/**
 * 
 * AiCiA - Android IRC Client
 * 
 * Copyright (C)2010 GORRY.
 * 
 */

package net.gorry.aicia;

import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import android.content.Intent;
import android.net.Uri;
import android.os.RemoteException;
import android.text.SpannableStringBuilder;
import android.util.Log;
import android.view.KeyEvent;

/**
 * @author GORRY
 *
 */
public class DoMain {
	private static final String TAG = "DoMain";
	private static final boolean V = false;
	private final ActivityMain me;
	private IIRCService iIRCService;

	/**
	 * コンストラクタ
	 * @param a アクティビティインスタンス
	 */
	DoMain(final ActivityMain a) {
		me = a;
	}

	/**
	 * iIRCServiceの登録
	 * @param i iIRCService
	 */
	public void setIIRCService(final IIRCService i) {
		iIRCService = i;
	}

	/**
	 * IRCサーバリスト設定へ遷移
	 */
	public void doServerListConfig() {
		if (V) Log.v(TAG, "doServerListConfig()");
		final Intent intent = new Intent(
				me,
				ActivityIRCServerListConfig.class
		);
		me.startActivity(intent);
	}

	/**
	 * システム設定へ遷移
	 */
	public void doSystemConfig() {
		if (V) Log.v(TAG, "doSystemConfig()");
		final Intent intent = new Intent(
				me,
				ActivitySystemConfig.class
		);
		final boolean isLandscape = SystemConfig.getOrientation();
		intent.putExtra("islandscape", isLandscape);
		me.startActivityForResult(intent, ActivityMain.ACTIVITY_SYSTEMCONFIG);
	}

	/**
	 * IRCサーバ自動接続
	 */
	public void doConnectAuto() {
		if (V) Log.v(TAG, "doConnectAuto()");
		try {
			iIRCService.connectAuto();
		}
		catch(final RemoteException e) {
			// エラー
		}
	}

	/**
	 * 完全終了
	 */
	public void doShutdown() {
		if (V) Log.v(TAG, "doShutdown()");
		(me).runOnUiThread(new Runnable(){ public void run() {
			final AwaitAlertDialogOkCancel dlg = new AwaitAlertDialogOkCancel(me);
			dlg.setTitle(me.getString(R.string.activitymain_java_confirmfinish_title));
			dlg.setMessage(me.getString(R.string.activitymain_java_confirmfinish));
			dlg.create();
			new Thread(new Runnable() {	public void run() {
				// 終了してよいか確認
				final int result = dlg.show();
				switch (result) {
					case AwaitAlertDialogBase.OK:
					{
						(me).runOnUiThread(new Runnable(){ public void run() {
							ActivityMain.mShutdownServiceOnDestroy = true;
							me.finish();
						} });
						break;
					}
					case AwaitAlertDialogBase.CANCEL:
					default:
						break;
				}
			} }).start();
		} });
	}

	/**
	 * チャンネル変更
	 * @param serverName サーバ設定名
	 * @param channel チャンネル名
	 * @param showToast トーストを表示するならtrue
	 */
	public synchronized void doChangeChannel(final String serverName, final String channel, final boolean showToast) {
		ActivityMain.mCurrentServerName = new String(serverName);
		ActivityMain.mCurrentChannel = new String(channel);
		try {
			iIRCService.setCurrentServerName(serverName);
			iIRCService.setCurrentChannel(serverName, channel);
			iIRCService.clearChannelUpdated(serverName, channel);
			ActivityMain.mTopic = iIRCService.getTopic(serverName, channel);
			if (showToast) {
				me.myShortToastShow(IRCMsg.serverChannelToast(serverName, channel));
			}
			final SpannableStringBuilder chLog = new SpannableStringBuilder(iIRCService.getSpanChannelLog(serverName, channel));
			ActivityMain.layout.mMainLogWindow.setMessage(chLog);
			me.updateTitle();
		} catch (final RemoteException e) {
			e.printStackTrace();
		}
	}

	/**
	 * 次のチャンネルへ移動
	 * @param dir 方向
	 * @param mode モード
	 * @param showToast トーストを表示
	 * @return チャンネル移動が成功したらtrue
	 */
	public synchronized boolean doChangeNextChannel(final int dir, final int mode, final boolean showToast) {
		try {
			final boolean result = iIRCService.changeNextChannel(dir, mode);
			if (result) {
				final String serverName = iIRCService.getCurrentServerName();
				final String channel = iIRCService.getCurrentChannel(serverName);
				ActivityMain.mCurrentServerName = serverName;
				ActivityMain.mCurrentChannel = channel;
				ActivityMain.mTopic = iIRCService.getTopic(serverName, channel);
				iIRCService.clearChannelUpdated(serverName, channel);
				if (showToast) {
					me.myShortToastShow(IRCMsg.serverChannelToast(serverName, channel));
				}
				final SpannableStringBuilder chLog = new SpannableStringBuilder(iIRCService.getSpanChannelLog(serverName, channel));
				ActivityMain.layout.mMainLogWindow.setMessage(chLog);
				me.updateTitle();
				return true;
			}
		} catch (final RemoteException e) {
			e.printStackTrace();
		}
		return false;
	}

	/**
	 * フラット化したサーバ名・チャンネルリストの取得
	 * @param flatServerName
	 * @param flatChannel
	 * @param flatChannelUpdated
	 * @return
	 */
	private int getFlatServerChannelList(final ArrayList<String> flatServerName, final ArrayList<String> flatChannel, final ArrayList<Boolean> flatChannelUpdated ) {
		int noFlat = 0;
		int select = -1;

		try {
			final String currentServerName = iIRCService.getCurrentServerName();
			if (currentServerName == null) return -1;
			final String currentChannel = iIRCService.getCurrentChannel(currentServerName);
			if (currentChannel == null) return -1;
			final String[] svList = iIRCService.getServerList();
			for (int i = 0; i < svList.length; i++) {
				final String[] chList = iIRCService.getChannelList(svList[i]);
				final boolean[] chuList = iIRCService
				.getChannelUpdatedList(svList[i]);
				for (int j = 0; j < chList.length; j++) {
					flatServerName.add(svList[i]);
					flatChannel.add(chList[j]);
					flatChannelUpdated.add(chuList[j]);
					if (svList[i].equalsIgnoreCase(currentServerName)
							&& chList[j].equalsIgnoreCase(currentChannel)) {
						select = noFlat;
					}
					noFlat++;
				}
			}
		} catch (final Exception e) {
			e.printStackTrace();
		}
		return select;
	}

	/**
	 * 全チャンネルから移動先を選択
	 */
	public void doSelectChannel() {
		(me).runOnUiThread(new Runnable(){ public void run() {
			// チャンネル一覧を得る
			final AwaitAlertDialogList dlg = new AwaitAlertDialogList(me);
			final ArrayList<String> flatServerName = new ArrayList<String>();
			final ArrayList<String> flatChannel = new ArrayList<String>();
			final ArrayList<Boolean> flatChannelUpdated = new ArrayList<Boolean>();
			final int currentId = getFlatServerChannelList(flatServerName, flatChannel, flatChannelUpdated);
			final int numFlat = flatServerName.size();
			if (numFlat == 0) {
				me.myShortToastShow(me.getString(R.string.activitymain_java_nochannel));
				return;
			}

			// メニューから選択
			for (int i=0; i<flatServerName.size(); i++) {
				String item;
				if (flatChannel.get(i).equals(IRCMsg.sSystemChannelName))
				{
					item = "[" + flatServerName.get(i) + "]";
					dlg.addItem(item, 1);
				} else {
					item = "  " + flatChannel.get(i);
					dlg.addItem(item, 2);
				}
			}
			dlg.setTitle(me.getString(R.string.activitymain_java_channellist));
			dlg.setSelection(currentId);
			dlg.setLongClickEnable(false);
			dlg.create();
			new Thread(new Runnable() {	public void run() {
				// 指定チャンネルへ移動
				final int result = dlg.show()-AwaitAlertDialogList.CLICKED;
				if ((0 <= result) && (result < numFlat)) {
					(me).runOnUiThread(new Runnable(){ public void run() {
						doChangeChannel(flatServerName.get(result), flatChannel.get(result), true);
					} });
				}
			} }).start();
		} });
	}

	/**
	 * 更新されたチャンネルから移動先を選択
	 * 移動先には「更新チャンネル」「システムチャンネル」が
	 * 必ず含まれる。
	 */
	public void doSelectUpdatedChannel()
	{
		(me).runOnUiThread(new Runnable(){ public void run() {

			// チャンネル一覧を得る
			final AwaitAlertDialogList dlg = new AwaitAlertDialogList(me);
			final ArrayList<String> flatServerName = new ArrayList<String>();
			final ArrayList<String> flatChannel = new ArrayList<String>();
			final ArrayList<Boolean> flatChannelUpdated = new ArrayList<Boolean>();
			int currentId = getFlatServerChannelList(flatServerName, flatChannel, flatChannelUpdated);
			int numFlat = flatServerName.size();
			int select = -1;
			int nItems = 0;
			boolean updated = false;
			for (int i=0; i<flatServerName.size(); i++) {
				if (flatChannelUpdated.get(i)) {
					updated = true;
					break;
				}
			}
			if (!updated) {
				me.myShortToastShow(me.getString(R.string.activitymain_java_notupdated));
				return;
			}

			// 更新済みチャンネル一覧を作る
			for (int i=0; i<flatServerName.size(); i++) {
				String item;
				final boolean isServerName = (flatChannel.get(i).equals(IRCMsg.sSystemChannelName));
				// if (flatChannelUpdated.get(i) || (i == currentId) || isServerName) {
				if (flatChannelUpdated.get(i) || isServerName) {
					if (isServerName) {
						item = "[" + flatServerName.get(i) + "]";
						dlg.addItem(item, 1);
					} else {
						item = "  " + flatChannel.get(i);
						dlg.addItem(item, 2);
					}
					if (i == currentId) {
						select = nItems;
					}
					nItems++;
				} else {
					// 未更新のチャンネルはリストから外す
					flatServerName.remove(i);
					flatChannel.remove(i);
					flatChannelUpdated.remove(i);
					numFlat--;
					if (currentId >= i) currentId--;
					i--;
				}
			}
			final int listSize = nItems;

			// 最後に「更新フラグのクリア」を追加
			dlg.addItem("[" + me.getString(R.string.activitymain_java_clearupdatedchannel)  + "]", 1);

			// メニューから選択
			dlg.setTitle(me.getString(R.string.activitymain_java_updatedchannellist));
			dlg.setSelection(select);
			dlg.setLongClickEnable(false);
			dlg.create();
			new Thread(new Runnable() {	public void run() {
				final int result = dlg.show()-AwaitAlertDialogList.CLICKED;
				if (result == listSize) {
					// 更新フラグのクリア
					try {
						iIRCService.clearChannelUpdatedAll();
						me.myShortToastShow(me.getString(R.string.activitymain_java_clearupdatedchannel_done));
					} catch (final RemoteException e) {
						e.printStackTrace();
					}
				} else if ((0 <= result) && (result < listSize)) {
					// 指定チャンネルへ移動
					(me).runOnUiThread(new Runnable(){ public void run() {
						doChangeChannel(flatServerName.get(result), flatChannel.get(result), true);
					} });
				}
			} }).start();
		} });
	}

	/**
	 * ダイアログ選択でユーザーリストを作成する
	 * @param userList 選択肢となるユーザーリスト
	 * @param titleResource ダイアログタイトルリソース
	 * @param isTIG TIGモードのときtrue
	 * @return 選択されたユーザーのリスト
	 */
	private String[] getUserListFromDialog(final String[] userList, final int titleResource, final boolean isTIG) {
		final Object o = new Object();
		// ユーザー名の取得
		final ArrayList<String> selectedUserList = new ArrayList<String>();
		if (userList.length == 0) {
			return null;
		}

		// ユーザー名リストを作成
		(me).runOnUiThread(new Runnable(){ public void run() {
			final AwaitAlertDialogListMulti dlg = new AwaitAlertDialogListMulti(me);
			for (int i=0; i<userList.length; i++) {
				if (isTIG) {
					dlg.addItem("@" + userList[i], 2, false);
				} else {
					dlg.addItem(userList[i], 2, false);
				}
			}
			dlg.setTitle(me.getString(titleResource));
			dlg.create();
			new Thread(new Runnable() {	public void run() {
				final int result = dlg.show();
				if (result == AwaitAlertDialogBase.CANCEL) {
					synchronized (o) {
						o.notifyAll();
					}
					return;
				}
				final Boolean[] selection = dlg.getSelection();
				for (int i=0; i<selection.length; i++) {
					if (selection[i]) {
						if (isTIG) {
							selectedUserList.add("@" + userList[i]);
						} else {
							selectedUserList.add(userList[i]);
						}
					}
				}
				synchronized (o) {
					o.notifyAll();
				}
			} }).start();
		}});
		synchronized (o) {
			try {
				o.wait();
			} catch (final InterruptedException e) {
				//
			}
		}
		if (selectedUserList.size() == 0 ) {
			return null;
		}
		final String[] retUserList = selectedUserList.toArray(new String[0]);
		return retUserList;
	}

	/**
	 * ダイアログ選択でユーザーを得る
	 * @param userList 選択肢となるユーザーリスト
	 * @param titleResource ダイアログタイトルリソース
	 * @param isTIG TIGモードのときtrue
	 * @return 選択されたユーザー
	 */
	private String getUserFromDialog(final String[] userList, final int titleResource, final boolean isTIG) {
		final Object o = new Object();
		// ユーザー名の取得
		final String[] selectedUser = new String[1];
		if (userList.length == 0) {
			return null;
		}
		selectedUser[0] = null;

		// ユーザー名リストを作成
		(me).runOnUiThread(new Runnable(){ public void run() {
			final AwaitAlertDialogList dlg = new AwaitAlertDialogList(me);
			for (int i=0; i<userList.length; i++) {
				if (isTIG) {
					dlg.addItem("@" + userList[i], 2);
				} else {
					dlg.addItem(userList[i], 2);
				}
			}
			dlg.setTitle(me.getString(titleResource));
			dlg.create();
			new Thread(new Runnable() {	public void run() {
				final int result = dlg.show();
				final int id = result - AwaitAlertDialogList.CLICKED;
				if ((result == AwaitAlertDialogBase.CANCEL) || (id < 0) || (id >= userList.length)) {
					synchronized (o) {
						o.notifyAll();
					}
					return;
				}
				if (isTIG) {
					selectedUser[0] = "@" + userList[id];
				} else {
					selectedUser[0] = userList[id];
				}
				synchronized (o) {
					o.notifyAll();
				}
			} }).start();
		}});
		synchronized (o) {
			try {
				o.wait();
			} catch (final InterruptedException e) {
				//
			}
		}
		if (selectedUser[0] == null) {
			return null;
		}
		return selectedUser[0];
	}

	/**
	 * カレントチャンネルユーザーからダイアログ選択でユーザーリストを作成する
	 * @param titleResource ダイアログタイトルリソース
	 * @param isTIG TIGモードのときtrue
	 * @param multiSelect ２人以上選択したいときはtrue
	 * @return 選択されたユーザーのリスト
	 */
	public String[] getUserListFromCurrentChannel(final int titleResource, final boolean isTIG, final boolean multiSelect) {
		// カレントチャンネルの取得
		boolean ok = false;
		String currentServerName = null;
		String currentChannel = null;
		try {
			currentServerName = iIRCService.getCurrentServerName();
			if (currentServerName != null) {
				currentChannel = iIRCService.getCurrentChannel(currentServerName);
				if (currentChannel != null) {
					ok = true;
				}
			}
		} catch (final RemoteException e) {
			e.printStackTrace();
		}
		if (!ok) {
			me.myShortToastShow(me.getString(R.string.activitymain_java_nouserlist));
			return null;
		}

		// ユーザー名の取得
		final String[] userList;
		try {
			userList = iIRCService.getUserList(currentServerName, currentChannel);
		} catch (final RemoteException e) {
			e.printStackTrace();
			return null;
		}
		if (userList.length == 0) {
			me.myShortToastShow(me.getString(R.string.activitymain_java_nouserlist));
			return null;
		}

		// 選択ユーザー名を取得
		if (multiSelect) {
			return getUserListFromDialog(userList, titleResource, isTIG);
		}
		final String retUser[] = new String[1];
		retUser[0] = getUserFromDialog(userList, titleResource, isTIG);
		return retUser;
	}

	/**
	 * カレントチャンネルTIGログからダイアログ選択でユーザーリストを作成する
	 * @param titleResource ダイアログタイトルリソース
	 * @param pickPrivUser 発言ユーザーを拾うときtrue
	 * @param pickTargetUser 発言内で対象になっているユーザーを拾うときtrue
	 * @param multiSelect ２人以上選択したいときはtrue
	 * @return 選択されたユーザーのリスト
	 */
	public String[] getUserListFromTIGLog(final int titleResource, final boolean pickPrivUser, final boolean pickTargetUser, final boolean multiSelect) {
		// カレントチャンネルの取得
		boolean ok = false;
		String currentServerName = null;
		String currentChannel = null;
		try {
			currentServerName = iIRCService.getCurrentServerName();
			if (currentServerName != null) {
				currentChannel = iIRCService.getCurrentChannel(currentServerName);
				if (currentChannel != null) {
					ok = true;
				}
			}
		} catch (final RemoteException e) {
			e.printStackTrace();
		}
		if (!ok) {
			me.myShortToastShow(me.getString(R.string.activitymain_java_nouserlist));
			return null;
		}

		// ログの取得
		final ArrayList<String> aUserList = new ArrayList<String>();
		String log = null;
		try {
			log = iIRCService.getSpanChannelLog(currentServerName, currentChannel).toString();
		} catch (final RemoteException e) {
			e.printStackTrace();
			return null;
		}
		if (log == null) {
			me.myShortToastShow(me.getString(R.string.activitymain_java_nouserlist));
			return null;
		}

		// ログを新しいほうから読んでユーザー名の取得
		final String regex = "[@][A-Za-z0-9_]+";
		final Pattern p = Pattern.compile(regex);
		final String regex2 = "^[^<]*<([^>]*)>.*$";
		final Pattern p2 = Pattern.compile(regex2);
		final String[] loglines = log.split("\n");
		for (int i=loglines.length-1; i>=0; i--) {
			if (pickPrivUser) {
				final Matcher m2 = p2.matcher(loglines[i]);
				if (m2.find()) {
					final String user = m2.replaceAll("$1");
					if (!aUserList.contains(user)) {
						aUserList.add(user);
					}
				}
			}
			if (pickTargetUser) {
				final Matcher m = p.matcher(loglines[i]);
				m.reset();
				while (m.find()) {
					final String user = m.group().substring(1);
					if (!aUserList.contains(user)) {
						aUserList.add(user);
					}
				}
			}
		}
		final int size = aUserList.size();
		final String[] userList = new String[size];
		for (int i=0; i<size; i++) {
			userList[i] = new String(aUserList.get(i));
		}

		// 選択ユーザー名を取得
		if (multiSelect) {
			return getUserListFromDialog(userList, titleResource, true);
		}
		final String retUser[] = new String[1];
		retUser[0] = getUserFromDialog(userList, titleResource, true);
		return retUser;

	}

	/**
	 * IRC用ユーティリティメニューの表示
	 */
	public void doUtilityIRC() {
		// 選択ユーザー名を挿入
		new Thread(new Runnable() {	public void run() {
			final String[] selectedUserList = getUserListFromCurrentChannel(
					R.string.activitymain_java_insertfromuserlist,
					false, true
			);
			if (selectedUserList != null) {
				(me).runOnUiThread(new Runnable(){ public void run() {
					for (int i=0; i<selectedUserList.length; i++) {
						EditAssist.insert(ActivityMain.layout.mInputBox, selectedUserList[i] + " ");
					}
					ActivityMain.layout.mInputBox.requestFocus();
				}});
			}
		} }).start();
	}

	/**
	 * ダイアログ選択でTypableMapを含むコマンドラインを作成する
	 * @param userList 選択肢となるユーザーリスト
	 * @param titleResource ダイアログタイトルリソース
	 * @param isTIG TIGモードのときtrue
	 * @return 選択されたユーザーのリスト
	 */
	private String getTIGCommandFromDialog(final int cmdno, final ArrayList<String> aTypableMapList, final ArrayList<String> aLineList, final int titleResource) {
		final Object o = new Object();
		final String[] retCmdLine = new String[1];
		retCmdLine[0] = null;

		// ユーザー名リストを作成
		(me).runOnUiThread(new Runnable(){ public void run() {
			final AwaitAlertDialogList dlg = new AwaitAlertDialogList(me);
			for (int i=0; i<aTypableMapList.size(); i++) {
				dlg.addItem(aTypableMapList.get(i) + ": " + aLineList.get(i), 2);
			}
			dlg.setTitle(me.getString(titleResource));
			dlg.setLongClickEnable(false);
			dlg.setListResource(R.layout.alertdialoglist_typablemap);
			dlg.create();
			new Thread(new Runnable() {	public void run() {
				final int result = dlg.show();
				if (result == AwaitAlertDialogBase.CANCEL) {
					synchronized (o) {
						o.notifyAll();
					}
					return;
				}
				final int id = result-AwaitAlertDialogList.CLICKED;
				switch (cmdno) {
					case 0:  // re
					{
						retCmdLine[0] = "re " + aTypableMapList.get(id) + " ";
						final String regex = "[@][A-Za-z0-9_]+";
						final Pattern p = Pattern.compile(regex);
						final Matcher m = p.matcher(aLineList.get(id));
						m.reset();
						while (m.find()) {
							final String user = m.group();
							retCmdLine[0] = retCmdLine[0] + user + " ";
						}
						break;
					}
					case 1:  // rt
						retCmdLine[0] = "rt " + aTypableMapList.get(id);
						break;
					case 2:  // qt
						retCmdLine[0] = "qt " + aTypableMapList.get(id) + " ";
						break;
					case 3:  // ort
						retCmdLine[0] = SystemConfig.TIG_oldRTCmd + " " + aTypableMapList.get(id) + " ";
						break;
				}
				synchronized (o) {
					o.notifyAll();
				}
			} }).start();
		}});
		synchronized (o) {
			try {
				o.wait();
			} catch (final InterruptedException e) {
				//
			}
		}
		return retCmdLine[0];
	}

	/**
	 * TIGコマンドを得る
	 * @param cmdno TIGコマンド番号（0=re, 1=rt, 2=qt, 4=ort）
	 * @param titleResource ダイアログタイトルリソース
	 * @return TIGコマンドライン
	 */
	public String getTIGCommand(final int cmdno, final int titleResource) {
		// カレントチャンネルの取得
		boolean ok = false;
		String currentServerName = null;
		String currentChannel = null;
		try {
			currentServerName = iIRCService.getCurrentServerName();
			if (currentServerName != null) {
				currentChannel = iIRCService.getCurrentChannel(currentServerName);
				if (currentChannel != null) {
					ok = true;
				}
			}
		} catch (final RemoteException e) {
			e.printStackTrace();
		}
		if (!ok) {
			me.myShortToastShow(me.getString(R.string.activitymain_java_nouserlist));
			return null;
		}

		// ログの取得
		final ArrayList<String> aLineList = new ArrayList<String>();
		final ArrayList<String> aTypableMapList = new ArrayList<String>();
		String log = null;
		try {
			log = iIRCService.getSpanChannelLog(currentServerName, currentChannel).toString();
		} catch (final RemoteException e) {
			e.printStackTrace();
			return null;
		}
		if (log == null) {
			me.myShortToastShow(me.getString(R.string.activitymain_java_nouserlist));
			return null;
		}
		// ログを新しいほうから読んでtypableMap行の取得
		final String regex = "^.* \\(([a-z]*)\\)$";
		final Pattern p = Pattern.compile(regex);
		final String regex2 = " \\([a-z]*\\)$";
		final Pattern p2 = Pattern.compile(regex2);
		final String[] loglines = log.split("\n");
		for (int i=loglines.length-1; i>=0; i--) {
			final Matcher m = p.matcher(loglines[i]);
			if (m.find()) {
				final String tmap = m.replaceAll("$1");
				final Matcher m2 = p2.matcher(loglines[i]);
				final String line = m2.replaceAll("");
				aLineList.add(line);
				aTypableMapList.add(tmap);
			}
		}

		// 選択ラインのTIGコマンドを取得
		return getTIGCommandFromDialog(cmdno, aTypableMapList, aLineList, titleResource);
	}

	/**
	 * TIG用ユーティリティメニューの表示
	 */
	public void doUtilityTIG() {
		// チャンネル確認
		boolean ok = false;
		String currentServerName = null;
		String currentChannel = null;
		try {
			currentServerName = iIRCService.getCurrentServerName();
			if (currentServerName != null) {
				currentChannel = iIRCService.getCurrentChannel(currentServerName);
				if (currentChannel != null) {
					ok = true;
				}
			}
		} catch (final RemoteException e) {
			e.printStackTrace();
		}
		if (!ok) {
			me.myShortToastShow(me.getString(R.string.activitymain_java_nouserlist));
			return;
		}

		// TIG用ユーティリティメニューの作成
		(me).runOnUiThread(new Runnable(){ public void run() {
			final AwaitAlertDialogList dlg = new AwaitAlertDialogList(me);
			dlg.addItem(me.getString(R.string.activitymain_java_tigutility_cmd_reply), 1);
			dlg.addItem(me.getString(R.string.activitymain_java_tigutility_cmd_rt), 1);
			dlg.addItem(me.getString(R.string.activitymain_java_tigutility_cmd_qt), 1);
			dlg.addItem(me.getString(R.string.activitymain_java_tigutility_cmd_ort), 1);
			dlg.addItem(me.getString(R.string.activitymain_java_tigutility_cmd_user), 1);
			dlg.addItem(me.getString(R.string.activitymain_java_tigutility_cmd_privuserfromlog), 1);
			dlg.addItem(me.getString(R.string.activitymain_java_tigutility_cmd_targetuserfromlog), 1);
			dlg.addItem(me.getString(R.string.activitymain_java_tigutility_cmd_openuserweb), 1);
			dlg.addItem(me.getString(R.string.activitymain_java_tigutility_cmd_openuserwebfromlog), 1);
			dlg.setTitle(me.getString(R.string.activitymain_java_tigutility_title));
			dlg.setLongClickEnable(false);
			dlg.create();
			new Thread(new Runnable() {	public void run() {
				final int result = dlg.show();
				switch (result-AwaitAlertDialogList.CLICKED) {
					default:
						break;
					case 0:  // cmd_reply
					{
						// reコマンドを挿入
						final String cmd = getTIGCommand(
								0,
								R.string.activitymain_java_tigutility_cmd_reply
						);
						if (cmd != null) {
							(me).runOnUiThread(new Runnable(){ public void run() {
								EditAssist.insert(ActivityMain.layout.mInputBox, cmd);
							}});
						}
						break;
					}
					case 1:  // cmd_rt
					{
						// rtコマンドを挿入
						final String cmd = getTIGCommand(
								1,
								R.string.activitymain_java_tigutility_cmd_rt
						);
						if (cmd != null) {
							(me).runOnUiThread(new Runnable(){ public void run() {
								EditAssist.insert(ActivityMain.layout.mInputBox, cmd);
							}});
						}
						break;
					}
					case 2:  // cmd_qt
					{
						// qtコマンドを挿入
						final String cmd = getTIGCommand(
								2,
								R.string.activitymain_java_tigutility_cmd_qt
						);
						if (cmd != null) {
							(me).runOnUiThread(new Runnable(){ public void run() {
								EditAssist.insert(ActivityMain.layout.mInputBox, cmd);
							}});
						}
						break;
					}
					case 3:  // cmd_ort
					{
						// ortコマンドを挿入
						final String cmd = getTIGCommand(
								3,
								R.string.activitymain_java_tigutility_cmd_ort
						);
						if (cmd != null) {
							(me).runOnUiThread(new Runnable(){ public void run() {
								EditAssist.insert(ActivityMain.layout.mInputBox, cmd);
							}});
						}
						break;
					}
					case 4:  // cmd_user
					{
						// 選択ユーザー名を挿入
						final String[] selectedUserList = getUserListFromCurrentChannel(
								R.string.activitymain_java_insertfromuserlist_tiguser,
								true, true
						);
						if (selectedUserList != null) {
							(me).runOnUiThread(new Runnable(){ public void run() {
								for (int i=0; i<selectedUserList.length; i++) {
									EditAssist.insert(ActivityMain.layout.mInputBox, selectedUserList[i] + " ");
								}
							}});
						}
						break;
					}
					case 5:  // cmd_privuserfromlog
					{
						// ログ内の発言ユーザー名を挿入
						final String[] selectedUserList = getUserListFromTIGLog(
								R.string.activitymain_java_insertfromuserlist_tiguserlog,
								true, false, true
						);
						if (selectedUserList != null) {
							(me).runOnUiThread(new Runnable(){ public void run() {
								for (int i=0; i<selectedUserList.length; i++) {
									EditAssist.insert(ActivityMain.layout.mInputBox, selectedUserList[i] + " ");
								}
							}});
						}
						break;
					}
					case 6:  // cmd_targetuserfromlog
					{
						// ログ内の対象ユーザー名を挿入
						final String[] selectedUserList = getUserListFromTIGLog(
								R.string.activitymain_java_insertfromuserlist_tiguserlog,
								false, true, true
						);
						if (selectedUserList != null) {
							(me).runOnUiThread(new Runnable(){ public void run() {
								for (int i=0; i<selectedUserList.length; i++) {
									EditAssist.insert(ActivityMain.layout.mInputBox, selectedUserList[i] + " ");
								}
							}});
						}
						break;
					}
					case 7:  // cmd_openuserweb
					{
						// チャンネル内ユーザーのTwitterサイトを開く
						final String[] selectedUserList = getUserListFromCurrentChannel(
								R.string.activitymain_java_openuserweb_tiguser,
								true, false
						);
						if ((selectedUserList != null) && (selectedUserList[0] != null)) {
							doOpenTwitterUserWeb(selectedUserList[0]);
						}
						break;
					}
					case 8:  // cmd_openuserwebfromlog
					{
						// ログ内ユーザーのTwitterサイトを開く
						final String[] selectedUserList = getUserListFromTIGLog(
								R.string.activitymain_java_openuserweb_tiguserlog,
								true, true, false
						);
						if ((selectedUserList != null) && (selectedUserList[0] != null)) {
							doOpenTwitterUserWeb(selectedUserList[0]);
						}
						break;
					}
				}
				(me).runOnUiThread(new Runnable(){ public void run() {
					ActivityMain.layout.mInputBox.requestFocus();
				}});
			} }).start();
		}});
	}

	/**
	 * Twitterユーザーwebの表示
	 * @param userName ユーザー名
	 */
	public void doOpenTwitterUserWeb(final String userName) {
		final String name = userName.substring(1);
		final Intent intent = new Intent();
		intent.setAction(Intent.ACTION_VIEW);
		intent.setData(Uri.parse("http://twitter.com/" + name));
		me.startActivity(intent);
	}

	/**
	 * AiCiAヘルプの表示
	 */
	public void doShowHelp() {
		final Intent intent = new Intent();
		intent.setAction(Intent.ACTION_VIEW);
		intent.setData(Uri.parse("http://gorry.hauN.org/android/aicia/help/"));
		me.startActivity(intent);
	}

	/**
	 * ユーティリティの起動
	 */
	public void doUtility() {
		if (ActivityMain.mCurrentServerName != null) {
			boolean isTIGMode = false;
			try {
				isTIGMode = iIRCService.isTIGMode(ActivityMain.mCurrentServerName);
			} catch (final RemoteException e) {
				e.printStackTrace();
			}
			if (isTIGMode) {
				doUtilityTIG();
				return;
			}
			doUtilityIRC();
		}
	}

	/**
	 * スクリーンオブジェクトのショートカット処理
	 * @param keyCode キーコード
	 * @param event イベント
	 * @return キー処理を行ったらtrue
	 */
	public boolean doShortcutKey(final int keyCode, final KeyEvent event) {
		if (event.isAltPressed()) {
			switch (keyCode) {
				case KeyEvent.KEYCODE_MENU:
					if (event.isShiftPressed()) {
						doUtilityIRC();
					} else {
						doUtility();
					}
					return true;

				case KeyEvent.KEYCODE_SEARCH:
					if (event.isShiftPressed()) {
						doSelectUpdatedChannel();
					} else {
						doSelectChannel();
					}
					return true;

				case KeyEvent.KEYCODE_DPAD_UP:
					if (event.isShiftPressed()) {
						final boolean changed = doChangeNextChannel(-1, 1, false);
						if (!changed) {
							me.myShortToastShow(me.getString(R.string.activitymain_java_notupdated));
						}
					} else {
						final boolean changed = doChangeNextChannel(-1, 0, false);
						if (!changed) {
							me.myShortToastShow(me.getString(R.string.activitymain_java_notchanged));
						}
					}
					ActivityMain.layout.mInputBox.requestFocus();
					return true;
				case KeyEvent.KEYCODE_DPAD_DOWN:
					if (event.isShiftPressed()) {
						final boolean changed = doChangeNextChannel(1, 1, false);
						if (!changed) {
							me.myShortToastShow(me.getString(R.string.activitymain_java_notupdated));
						}
					} else {
						final boolean changed = doChangeNextChannel(1, 0, false);
						if (!changed) {
							me.myShortToastShow(me.getString(R.string.activitymain_java_notchanged));
						}
					}
					ActivityMain.layout.mInputBox.requestFocus();
					return true;
			}
			if (EditAssist.cutCopyPaste(me, ActivityMain.layout.mInputBox, keyCode, event)) {
				return true;
			}
		}

		return false;
	}


}
